"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var R = require("ramda");
var Promise = require("bluebird");
var pdfjsLib = require("pdfjs-dist");
var ts_smart_logger_1 = require("ts-smart-logger");
var util_1 = require("../../../util");
var UniversalPdfPlugin = /** @class */ (function () {
    /**
     * @stable [14.11.2018]
     * @param {string} workerSrc
     * @param {() => HTMLCanvasElement} canvasResolver
     * @param {() => void} onSuccess
     * @param {(error: AnyT) => void} onError
     */
    function UniversalPdfPlugin(workerSrc, canvasResolver, onSuccess, onError) {
        this.workerSrc = workerSrc;
        this.canvasResolver = canvasResolver;
        this.onSuccess = onSuccess;
        this.onError = onError;
        this.page = 1;
        this.scale = 1;
        this.onLoadError = this.onLoadError.bind(this);
        this.onLoadPage = this.onLoadPage.bind(this);
        this.onLoad = this.onLoad.bind(this);
        pdfjsLib.GlobalWorkerOptions.workerSrc = workerSrc;
    }
    /**
     * @stable [14.11.2018]
     * @param {number} page
     * @returns {IUniversalPdfPlugin}
     */
    UniversalPdfPlugin.prototype.setPage = function (page) {
        this.page = page;
        return this;
    };
    /**
     * @stable [14.11.2018]
     * @param {string} src
     * @returns {IUniversalPdfPlugin}
     */
    UniversalPdfPlugin.prototype.setSrc = function (src) {
        this.src = src;
        return this;
    };
    /**
     * @stable [14.11.2018]
     * @param {number} scale
     * @returns {IUniversalPdfPlugin}
     */
    UniversalPdfPlugin.prototype.setScale = function (scale) {
        this.scale = scale;
        return this;
    };
    /**
     * @stable [14.11.2018]
     */
    UniversalPdfPlugin.prototype.loadDocument = function () {
        this.cancel();
        if (!this.src) {
            UniversalPdfPlugin.logger.warn('[$UniversalPdfPlugin][loadDocument] The src is not defined!');
            return;
        }
        var loadTask = pdfjsLib.getDocument(this.src);
        this.loadTask = Promise.resolve(loadTask.promise)
            .then(this.onLoad, this.onLoadError);
    };
    /**
     * @stable [14.11.2018]
     */
    UniversalPdfPlugin.prototype.refreshPage = function () {
        if (!this.hasLoadedDocument) {
            UniversalPdfPlugin.logger.warn('[$UniversalPdfPlugin][refreshPage] The document is not defined!');
            return;
        }
        this.loadPageTask = Promise.resolve(this.loadedDocument.getPage(this.page))
            .then(this.onLoadPage);
    };
    Object.defineProperty(UniversalPdfPlugin.prototype, "hasLoadedDocument", {
        /**
         * @stable [14.11.2018]
         * @returns {boolean}
         */
        get: function () {
            return !R.isNil(this.loadedDocument);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(UniversalPdfPlugin.prototype, "numPages", {
        get: function () {
            return this.loadedDocument.numPages;
        },
        enumerable: true,
        configurable: true
    });
    /**
     * @stable [14.11.2018]
     */
    UniversalPdfPlugin.prototype.cancel = function () {
        this.cancelTask();
        this.cancelPageTask();
        this.loadTask = null;
        this.loadPageTask = null;
        this.loadedDocument = null;
    };
    /**
     * @stable [14.11.2018]
     * @param {IPdfViewerDocument} pdf
     * @returns {IPdfViewerDocument}
     */
    UniversalPdfPlugin.prototype.onLoad = function (pdf) {
        this.loadedDocument = pdf;
        if (util_1.isFn(this.onSuccess)) {
            this.onSuccess();
        }
        this.refreshPage();
        return pdf;
    };
    /**
     * @stable [14.11.2018]
     * @param {IPdfViewerPage} page
     * @returns {IPdfViewerPage}
     */
    UniversalPdfPlugin.prototype.onLoadPage = function (page) {
        var renderArea = this.renderArea;
        var viewportParent = util_1.toJqEl(renderArea.parentElement);
        var w = viewportParent.width();
        var h = viewportParent.height();
        var unscaledViewport = page.getViewport(1);
        var actualScale = Math.min((h / unscaledViewport.height), (w / unscaledViewport.width));
        var viewport = page.getViewport(actualScale);
        var canvasContext = renderArea.getContext('2d');
        renderArea.height = h;
        renderArea.width = w;
        page.render({ canvasContext: canvasContext, viewport: viewport });
        return page;
    };
    /**
     * @stable [14.11.2018]
     * @param {AnyT} error
     * @returns {AnyT}
     */
    UniversalPdfPlugin.prototype.onLoadError = function (error) {
        UniversalPdfPlugin.logger.error("[$UniversalPdfPlugin][onLoadError] Error:", error);
        if (util_1.isFn(this.onError)) {
            this.onError(error);
        }
        return error;
    };
    /**
     * @stable [14.11.2018]
     */
    UniversalPdfPlugin.prototype.cancelTask = function () {
        if (this.loadTask && this.loadTask.isPending()) {
            this.loadTask.cancel();
            UniversalPdfPlugin.logger.warn("[$UniversalPdfPlugin][cancelTask] The pdf task has been cancelled.");
        }
    };
    /**
     * @stable [14.11.2018]
     */
    UniversalPdfPlugin.prototype.cancelPageTask = function () {
        if (this.loadPageTask && this.loadPageTask.isPending()) {
            this.loadPageTask.cancel();
            UniversalPdfPlugin.logger.warn("[$UniversalPdfPlugin][cancelPageTask] The pdf page task has been cancelled.");
        }
    };
    Object.defineProperty(UniversalPdfPlugin.prototype, "renderArea", {
        /**
         * @stable [14.11.2018]
         * @returns {HTMLCanvasElement}
         */
        get: function () {
            return this.canvasResolver();
        },
        enumerable: true,
        configurable: true
    });
    UniversalPdfPlugin.logger = ts_smart_logger_1.LoggerFactory.makeLogger('UniversalPdfPlugin');
    return UniversalPdfPlugin;
}());
exports.UniversalPdfPlugin = UniversalPdfPlugin;
//# sourceMappingURL=universal-pdf.plugin.js.map