"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var definition_1 = require("../definition");
var di_services_1 = require("../di/di.services");
var type_1 = require("./type");
var object_1 = require("./object");
var sort_1 = require("./sort");
var cond_1 = require("./cond");
// Examples:
//   1) * * * * * *
//      Each minute
//
//   2) */10 9-17 1 * *
//      At every 10th minute past every hour from 9 through 17 on day-of-month 1
//
//   3) */10,*/2,1-2 9-17 1 * *
//      At every 10th minute, every 2nd minute, and every minute from 1 through 2 past
//      every hour from 9 through 17 on day-of-month 1
//
//   4)  1-59/2 * * * *
//       At every 2nd minute from 1 through 59 ("Выполняется по нечётным минутам")
/**
 * @stable [15.11.2019]
 *
 *  * * * * * * *
 *  | | | | | |
 *  | | | | | +-- Year              (range: 1900-3000)
 *  | | | | +---- Day of the Week   (range: 1-7, 1 standing for Monday)
 *  | | | +------ Month of the Year (range: 1-12)
 *  | | +-------- Day of the Month  (range: 1-31)
 *  | +---------- Hour              (range: 0-23)
 *  +------------ Minute            (range: 0-59)
 *
 * @param {string} expression
 * @param {Date} from
 * @param {Date} to
 * @returns {ICronEntity}
 */
exports.fromCronExpression = function (expression, from, to) {
    if (!object_1.isObjectNotEmpty(expression)) {
        return {
            daysOfMonths: [],
            daysOfWeeks: [],
            hours: [],
            minutes: [],
            months: [],
            years: [],
        };
    }
    var parts = expression.replace(/ {2,}/g, ' ').split(' ');
    var minutesPart = parts[0];
    var hoursPart = parts[1];
    var daysOfMonthPart = parts[2];
    var monthsOfYearPart = parts[3];
    var daysOfWeekPart = parts[4];
    var yearsPart = parts[6];
    return {
        daysOfMonths: fromCronPart(daysOfMonthPart, definition_1.CronPartTypesEnum.DAY_OF_MONTH, from, to),
        daysOfWeeks: fromCronPart(daysOfWeekPart, definition_1.CronPartTypesEnum.DAY_OF_WEEK, from, to),
        hours: fromCronPart(hoursPart, definition_1.CronPartTypesEnum.HOUR, from, to),
        minutes: fromCronPart(minutesPart, definition_1.CronPartTypesEnum.MINUTE, from, to),
        months: fromCronPart(monthsOfYearPart, definition_1.CronPartTypesEnum.MONTH_OF_YEAR, from, to),
        years: fromCronPart(yearsPart, definition_1.CronPartTypesEnum.YEAR, from, to),
    };
};
/**
 * @stable [15.11.2019]
 * @param {string} cronPart
 * @param {CronPartTypesEnum} partType
 * @param {Date} from
 * @param {Date} to
 * @returns {number[]}
 */
var fromCronPart = function (cronPart, partType, from, to) {
    if (!object_1.isObjectNotEmpty(cronPart)) {
        return [];
    }
    var useRealDates = object_1.isObjectNotEmpty(from) || object_1.isObjectNotEmpty(to);
    var nc = di_services_1.getNumberConverter();
    var availableRanges = definition_1.CRON_PART_TYPE_RANGES[partType];
    var values = [];
    cronPart.replace(/ /g, '').split(',').forEach(function (item) {
        var repeatingParts = item.split('/');
        var ranges = repeatingParts[0].split('-');
        var isRange = ranges.length > 1;
        var period = cond_1.orNull(repeatingParts.length > 1, function () { return nc.number(repeatingParts[1]); });
        var isPeriodPresent = type_1.isNumber(period);
        var isFullPeriod = ranges[0] === '*';
        var start = isFullPeriod ? availableRanges[0] : nc.number(ranges[0]);
        var end = isFullPeriod ? availableRanges[1] : (isRange ? nc.number(ranges[1]) : start);
        var j = start;
        for (var i = start; i <= end; i++) {
            if (isPeriodPresent) {
                if (i === start || i === j) {
                    values.push(i);
                    j += period;
                }
            }
            else {
                values.push(i);
            }
        }
    });
    return Array.from(new Set(values)).sort(sort_1.NUMBER_COMPARATOR);
};
/**
 * @stable [15.11.2019]
 * @param {number} value
 * @param {CronPartTypesEnum} partType
 * @returns {boolean}
 */
var ifCronPartApplicable = function (value, partType) {
    var availableRanges = definition_1.CRON_PART_TYPE_RANGES[partType];
    return value >= availableRanges[0] && value <= availableRanges[1];
};
//# sourceMappingURL=cron.js.map